/************************************************************************************************

 Author        : ADI - Apps            www.analog.com/AduC7060

 Date          : November 2007

 File          : I2C_Slave.c

 Hardware      : ADuC706x

 Description   : This I2C Slave example shows how to initialise the I2C peripheral in Slave mode.
 			 In conjunction with the I2C_Master project and the I2CSerial.exe
			 application, this slave behaves as a 256byte EEPROM device.
		
*************************************************************************************************/
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define BIT16 0x10000
#define BIT22 0x400000
#define BIT24 0x1000000
#define BIT30 0x40000000

#include<aduc7060.h>
#include "common.h"
void delay(int); 


BYTE A0h[256], A2h[256];
BYTE i2c_first = TRUE;

unsigned char ucTxCnt = 0;
unsigned char ucRxCnt =0;
unsigned char ucEmptyCnt = 0;
unsigned char ucStopCnt =0;
unsigned char ucTestExtraRead = 0;
unsigned char ucFirstDataWrite = 0;

static BYTE byte_addr0 = 0x00;
static BYTE byte_addr2 = 0x00;
static BYTE wr_byte_addr0 = 0;

int main(void)
{
    unsigned int i;
	POWKEY1 = 0x1;
	POWCON0 = 0x78;		   // Set core to max CPU speed of 10.24Mhz
	POWKEY2 = 0xF4;
	GP1DAT  = BIT30;		// Configure P1.6 as an output
	// Configure P0.1 and P0.3 for I2C mode
	GP0CON0 = BIT4 + BIT12; // Select SPI/I2C alternative function for P0.1 & P0.3
	GP0KEY1 = 0x7;		   // Write to GP0KEY1
	GP0CON1	= BIT1;		   // Select I2C functionality for P0.1 & P0.3
	GP0KEY2 = 0x13;		   // Write to GP0KEY2

	// Enable I2C Slave mode, Slave ID and interrupt sources
	I2CID0  = 0xA0;			// Configure Slave's I2C network address (8-bit)
//	I2CID1  = 0xA0;			// Configure Slave's I2C network address (10-bit)
//	I2CID0  = 0xF2;			// Configure Slave's I2C network address (10-bit)
	I2CFSTA = BIT8;			// Flush Slave Tx FIFO
	I2CFSTA &= ~BIT8;
	I2CSCON = BIT0 + // BIT1 (10-bit address)    // Enable I2C Slave + Enable Rx interrupt +
			+ BIT9 + BIT10  // Enable Rx interrupt + Enable Tx interrupt +
			+ BIT8;		// Enable "Stop-after-Start-Detected" IRQ.
	IRQEN = BIT16; 			// Enable I2C Slave interrupts

	// Initialize table    
    for(i=0;i<256;i++){
        A0h[i]=i;
        A2h[i]=(255-i);
    }

    i2c_first = TRUE;
	while (1)
	{
	}
   return 0x00;
}

void delay (int length)
{
	while (length >0)
    	length--;
}
void IRQ_Handler(void) __irq
{
	unsigned long IRQSTATUS = 0;
	unsigned int I2CSSTATUS = 0;

	IRQSTATUS = IRQSTA;	   // Read off IRQSTA register

	if ((IRQSTATUS & BIT16) == BIT16)	//If I2C Slave interrupt source
	{
	   I2CSSTATUS = I2CSSTA;
	   if ((I2CSSTATUS & BIT3) == BIT3) // If I2C Slave Rx IRQ
	   {
			                         // first slave-receiver INT?
   	   		if(i2c_first == TRUE)                          // first slave-receiver INT?
			{   
			   // Do we need to flush the FIFOs ????I2C0FSTA |= 0x100;
			   byte_addr0 = I2CSRX;   // get byte address
		
			   wr_byte_addr0 = byte_addr0; 
			   I2CSTX = A0h[byte_addr0];	 // 2007 Uncommented
			   byte_addr0++;                
			   i2c_first = FALSE;
			   ucFirstDataWrite = 1;
		 	}
		   else
		 	{
				if (ucFirstDataWrite == 1)
				{
				 	 byte_addr0 -= 1;
				}
			    A0h[wr_byte_addr0] = I2CSRX;               // get byte data
			    // Do we need to flush the FIFOs ????I2C0FSTA |= 0x100;
			    byte_addr0 = I2CSRX;   // get byte address
			    byte_addr0++;						   // increment address
				wr_byte_addr0++;
			    ucFirstDataWrite = 0;  
			/*	// Test routine for clock stretching (I2CSCON.6)
				GP1DAT ^= BIT22;	// Toggle LED on Evaluation board (P1.6)
				I2CSCON |= BIT6;
				delay(100);
				I2CSCON &= 0xFFBF; 
				GP1DAT ^= BIT22;	// Toggle LED on Evaluation board (P1.6)  
				// test routine for I2CSCON.7
				if (wr_byte_addr0 == 3)
					I2CSCON |= BIT7; */                       
		    }
	   }
	   if ((I2CSSTATUS & BIT2) == BIT2) // If I2C Master Tx IRQ
	   {
			I2CSTX = A0h[byte_addr0++]; // set TX data + increment byte address
       }   
	
	   if ((I2CSSTATUS & BIT10) == BIT10) // If I2C Stop after Start condition detected IRQ
	   {
		  	i2c_first = TRUE;
			I2CFSTA = BIT8;			// Flush Slave Tx FIFO
			I2CFSTA &= ~BIT8;
			ucStopCnt++; 

	   }
	   if ((I2CSSTATUS & BIT13) == BIT13) // If I2C Repeated Start detected IRQ
	   {
			i2c_first = TRUE;	// Reset counter Array pointer.
	   }
	}
}


